package POWER::Utils;

use version; $VERSION = qv('0.5.0');    # update POD & Changes & README

require v5.6.0;     # need flush open filehandles before fork() and exec()

use warnings;
use strict;
use Carp;

use Scalar::Util qw( weaken );
use POSIX qw( :signal_h );

use Perl6::Export::Attrs;


sub clock_monotonic :Export(:DEFAULT);
use Inline C => Config => LIBS => '-lrt';
use Inline C => <<'END_OF_C';
#define __NR_timer_create   259
#define __NR_clock_gettime  (__NR_timer_create+6)
#define SYS_clock_gettime   __NR_clock_gettime
#include <time.h>
#include <sys/syscall.h>
#include <unistd.h>
double clock_monotonic() {
    struct timespec ts;
    // use syscall because glibc-2.3.3 doesn't support CLOCK_MONOTONIC yet
    if (syscall(SYS_clock_gettime, CLOCK_MONOTONIC, &ts) == -1)
        croak("clock_gettime: %s\n", strerror(errno));
    return ts.tv_sec+ts.tv_nsec/1e9;
}
END_OF_C


my $STDREF = qr{SCALAR|ARRAY|HASH|CODE|REF|GLOB|LVALUE};
sub run_callback :Export(:DEFAULT) {
    croak  q{usage: run_callback( CB [, METHOD [, @ARGS]] )}    if @_ < 1;
    my ($cb, $method) = (shift, shift);
    my $cb_type 
        = !ref($cb)                         ? 'CLASS' 
        : ref($cb) eq 'CODE'                ? 'CODE'
        : ref($cb) !~ m{\A$STDREF\z}xmso    ? 'OBJECT'
        :                                     undef
        ;
    if ($cb_type eq 'CLASS' || $cb_type eq 'OBJECT') {
        $cb->$method(@_);
    }
    elsif ($cb_type eq 'CODE') {
        $cb->(@_);
    }
    else {
        croak qq{run_callback: wrong CB $cb};
    }
    return;
}

sub weak_cb :Export {
    my ($obj, $method, @p) = @_;
    weaken(my $this = $obj);
    return sub { $this && $this->$method(@p, @_) };
}

# from "Perl Cookbook"
my @RandCh = ( 'A'..'Z', 'a'..'z', 0..9, '_' );
sub RandCh :Export(:DEFAULT) {
    croak 'usage: RandCh( LENGTH [, CHARS] )' if !@_;
    my $length = shift;
    croak 'RandCh: LENGTH must be positive' if !defined $length || $length <= 0;
    my @chars  = @_ ? @_ : @RandCh;
    return join("", @chars[ map { rand @chars } 1 .. $length ]);
}


sub Cat :Export(:DEFAULT) {
    croak 'usage: Cat( FILENAME )' if @_ != 1;
    my ($filename) = @_;
    open my $f, '<', $filename or croak "open: $!";
    local $/ if !wantarray;
    return <$f>;
}


sub Echo :Export(:DEFAULT) {
    croak 'usage: Echo( MODE_FILENAME, LIST )' if @_ < 2;
    my $filename = shift;
    open my $f, $filename or croak "open: $!";
    croak 'Echo: Bad file descriptor' if !defined syswrite($f, q{});    # XXX
    @_ = map {"$_\n"} @_ if "@_" !~ /\n/;
    print {$f} join(q{}, @_);
    return;
}


sub Chomp :Export(:DEFAULT) {
    croak 'usage: Chomp( LIST )' if !@_;
    my @lines = @_;
    chomp @lines;
    s/\r\z//xms for @lines;
    return wantarray ? @lines : $lines[0];
}


sub Eval :Export(:DEFAULT) {
    croak 'usage: Eval( SCALAR [, EVALNAME] )' if !(1 <= @_ && @_ <= 2);
    my ($code, $eval_name) = @_;
    $eval_name ||= '[use Eval(SCALAR, EVALNAME) to see EVALNAME here]';
    open my $f, '<', \$code or croak "open: $!";
    local @INC = ( sub {shift @INC; $f}, @INC );
    do $eval_name;
}


my %Enc = (
    qw( & &amp;   > &gt;   < &lt;   " &quot; ),
    map {chr, "&#$_;"} 0 .. 8, 11 .. 31, 127, 139, 155,
);
sub Enc :Export(:DEFAULT) {
    croak 'usage: Enc( SCALAR )' if @_ != 1;
    my ($html) = @_;
    croak 'Enc: $_[0] undefined' if !defined $html;
    $html =~ s/([\&\>\<\"\0-\10\13-\37\177])/$Enc{$1}/gs;
    return $html;
}


my %UnEnc = reverse 
    qw( & &amp;   > &gt;   < &lt;   " &quot; ),
    map({chr, "&#$_;"} 0 .. 255),
    map({chr, sprintf("&#x%02x;",$_)} 0 .. 255),
    ;
sub UnEnc :Export(:DEFAULT) {
    croak 'usage: UnEnc( SCALAR )' if @_ != 1;
    my ($html) = @_;
    croak 'UnEnc: $_[0] undefined' if !defined $html;
    $html =~ s/(&(?:#\d+|#x..|\w+);)/$UnEnc{lc($1)}/gs;
    return $html;
}


my %EncUri = map { chr($_), sprintf("%%%02X",$_) } 0..255;
sub EncUri :Export(:DEFAULT) {
    croak 'usage: EncUri( SCALAR [, REGEXP] )' if !(1 <= @_ && @_ <= 2);
    my ($to_escape, $regexp) = @_;
    croak 'EncUri: $_[0] undefined' if !defined $to_escape;
    $regexp ||= qr/[^\w *\@.-]/; 
    $to_escape =~ s/($regexp)/$EncUri{$1}/g;
    $to_escape =~ tr/ /+/ if ! $_[1] or " " =~ /$regexp/;   # XXX
    return $to_escape;
}


my %UnEncUri = map { sprintf("%%%02X",$_), chr($_) } 0..255;
sub UnEncUri :Export(:DEFAULT) {
    croak 'usage: UnEncUri( SCALAR )' if @_ != 1;
    my ($to_unescape) = @_;
    croak 'UnEncUri: $_[0] undefined' if !defined $to_unescape;
    $to_unescape =~ tr/+/ /;
    $to_unescape =~ s/(%[0-9A-Fa-f]{2})/$UnEncUri{uc($1)}/g;
    return $to_unescape;
}


sub _quote {
    my ($s) = @_;
    croak 'can\'t quote undefined value' if !defined $s;
    if ($s =~ / \s | ' | \A\z /xms) {
        $s =~ s/'/''/xmsg;
        $s = "'$s'";
    }
    return $s;
}

sub _unquote {
    my ($s) = @_;
    if ($s =~ s/\A'(.*)'\z/$1/xms) {
        $s =~ s/''/'/xmsg;
    }
    return $s;
}

sub quote_list :Export {
    return join q{ }, map {_quote($_)} @_;
}

sub unquote_list :Export {
    my ($s) = @_;
    return undef if !defined $s;
    my @w;
    while ($s =~ /\G ( [^'\s]+ | '[^']*(?:''[^']*)*' ) (?:\s+|\z)/xmsgc) {
        my $w = $1;
        push @w, _unquote($w);
    }
    return undef if $s !~ /\G\z/xmsg;
    return \@w;
}

sub unquote_hash :Export {
    my $w = unquote_list(@_);
    return $w && $#{$w} % 2 ? { @{$w} } : undef;
}


# -  SIGCHLD  sigprocmask(), ..   
#    SIGCHLD :
#   1) pid         SIGCHLD
#   2)     $?
#     ӣ-   ,     
#     SIGCHLD:
#       while (($pid = waitpid(-1,WNOHANG)) > 0) { ... }
#       .
#       POSIX.1 ,  SIGCHLD   
#            child   wait/waitpid
#         SIGCHLD    child  ݣ  
#        .       . :-/
# -    system() (.. fork+exec   
#    ),      
#      (    
#      ): SIGINT  SIGQUIT -    
#   ""  ,      
#   (    POSIX.1  system()).
# -  child      fd     child
#         fd,  fork   "" fd:
#   -    fd perl     close-on-exec
#       fd <= $^F.   fd     exec.
#   - M  fd,  ulimit (   ̣?):
#       POSIX::sysconf( &POSIX::_SC_OPEN_MAX );
#   -      /proc/$$/fd/.
#   -  fd  POSIX::close().
# -  child  /  ,    print/close
#     (,   STDIN),  :
#       local $SIG{PIPE} = 'IGNORE';
# -  fork  flush()  fd     child,
#          fd.   fork
#   ģ exec,        stdio  flush()
#     exec -     flush()  fork()  .
#       Beginning with v5.6.0, Perl will attempt to flush all files
#       opened for output before forking the child process.
#       Beginning with v5.6.0, Perl will attempt to flush all files
#       opened for output before the exec.
# !    (fork)   (exec) :
#                                       fork    exec
#         SIGALRM      -       +
#                     -       +
#                          +       +
#        (pending)      -       +
#       close-on-exec               +        
#                          +         close-on-exec
# - .. child    ,   'DEFAULT'/'IGNORE'
#   ,   exec   .
#
# NOTE:  child  STD{IN/OUT/ERR},    
#        parent,  STD{IN/OUT/ERR}  PIPE  SOCKET (.
#         CGI  apache),      
#       PIPE/SOCKET (apache)     parent  .
#           STD{IN/OUT/ERR}  / /dev/null
#        fork  exec  child.
#
#  : get/put  fh/cb:
#        get     => [\( undef,   $buf{out},  $buf{err}   )],
#        put     => [\( $stdin,  undef,      undef       )],
sub System :Export(:DEFAULT) {
    my ($opt_ref) = @_;
    croak 'Usage: System( \%opt )'  if @_ != 1;
    croak '{cmd} required'          if !defined $opt_ref->{cmd};
    croak "{$_}: unknown option"    for grep {
      !m/\A(?:real_cmd|cmd|fh|cb|method|timeout|foreground|inherit_signals)\z/
      } keys %$opt_ref;
    my $exception;
    #  .
    my $oldINT  = $SIG{INT}     || 'DEFAULT';
    my $oldQUIT = $SIG{QUIT}    || 'DEFAULT';
    if (!$opt_ref->{foreground}) {
        $SIG{INT}   = 'IGNORE';
        $SIG{QUIT}  = 'IGNORE';
    }
    my $sigchld = POSIX::SigSet->new( SIGCHLD );
    my $oldmask = POSIX::SigSet->new();
    sigprocmask(SIG_BLOCK, $sigchld, $oldmask)
        or die "sigprocmask: $!";
    # fork...
    my $pid = fork();
    if (!defined $pid) {
        die "fork: $!";
    }
    # *** CHILD
    elsif ($pid == 0) {
        #  .
        # restore PARENT signal layout
        if ($opt_ref->{inherit_signals}) {
            sigprocmask(SIG_SETMASK, $oldmask, undef)
                or die "sigprocmask: $!";
            $SIG{QUIT}  = $oldQUIT;
            $SIG{INT}   = $oldINT;
        }
        # restore DEFAULT signal layout
        else {
            sigprocmask(SIG_SETMASK, POSIX::SigSet->new(), undef)
                or die "sigprocmask: $!";
            $SIG{$_} = $_ eq 'CONT' ? 'IGNORE' : 'DEFAULT' for keys %SIG;
        }
        #  fd.
        if (defined $opt_ref->{fh}) {
            my @fh = @{ $opt_ref->{fh} };
            #    @fh  fd   0 .. $#fh 
            #   :
            # 1)    fd   <= $#fh .
            # 2)  @fh   fd (   
            #     > $#fh,       fd
            #      <= $#fh).
            # 3)   fd  fd   .
            # 4)   fd .. dup()   close-on-exec.
            my @fd_tmp;
            open $fd_tmp[$_], '+<', '/dev/null' or die "open: $!" for 0 .. $#fh;
            #  ,   mode    fh
            # ('<', '+>>', etc.),      open ,  
            #  fh  '&'   mode -,  
            #  fh (     mode
            # open -  ?!),     dup()
            #  dup2()    fd   mode.
            # NOTE  -   mode   
            #         /proc/$$/fd/,      :
            #       r/w/rw.   APPEND  TRUNC   .
            my @fd_dup = map { POSIX::dup(fileno($_)) or die "dup: $!" } @fh;
            POSIX::dup2($fd_dup[$_], $_) or die "dup2: $!"  for 0 .. $#fd_dup;
            POSIX::close($_)             or die "close: $!" for @fd_dup;
            # If user played with close-on-exec flag or $^F, or if @fh < $^F
            # then some additional fh may not be closed on exec. So we'll
            # manually close all unwanted filehandles and will not reckon
            # on close-on-exec flag.
            opendir my $dh, "/proc/$$/fd" or die "opendir: $!";
            my @unwanted = grep { m/\A\d+\z/ && $_ >= @fh } readdir $dh;
            POSIX::close($_) or die "close: $!" for @unwanted;
        }
        #  alarm.
        if (defined $opt_ref->{timeout}) {
            $SIG{ALRM} = 'DEFAULT';
            alarm($opt_ref->{timeout});
        }
        # exec...
        my @cmd = @{ $opt_ref->{cmd} };
        if (defined $opt_ref->{real_cmd}) {
            exec {$opt_ref->{real_cmd}} @cmd;
        } else {
            exec @cmd;
        }
        die "exec: $!";
    }
    # *** PARENT
    #  callback.
    if (defined $opt_ref->{cb}) {
        my $method = defined $opt_ref->{method} ? $opt_ref->{method} : 'SYSTEM';
        local $SIG{PIPE} = 'IGNORE';
        eval { run_callback($opt_ref->{cb}, $method, $pid) };
        if ($@) {
            $exception = $@;
            kill 9, $pid;
        }
    }
    #  exitcode.
    while (waitpid($pid, 0) < 0) {
        die "waitpid: $!" if !$!{EINTR};
    }
    my ($exitcode, $signal, $core) = ($? >> 8, $? & 127, $? & 128);
    #  .
    sigprocmask(SIG_SETMASK, $oldmask, undef)
        or die "sigprocmask: $!";
    $SIG{QUIT}  = $oldQUIT;
    $SIG{INT}   = $oldINT;
    # return...
    die $exception if $exception;
    return ($exitcode, $signal, $core);
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Utils - Bundle of useful subroutines.


=head1 VERSION

This document describes POWER::Utils version 0.5.0


=head1 SYNOPSIS

 use POWER::Utils;

 $sec = clock_monotonic();

 run_callback( \&sub );
 run_callback( 'CLASS', 'some_method' );
 run_callback( $obj, 'some_method', $foo, "bar" );

 $passwd = RandCh(8);

 $passwd_list = Cat("/etc/passwd");
 @passwd_list = Chomp(Cat("/etc/passwd"));

 Echo(">>$0.log", "Debug:", "message...") or die $!;

 Eval("use Any::Source::Filter; # filtered code ...");
 Eval("use Any::Source::Filter; # filtered code ...", 'some my code');

 pipe $pipe_r, $pipe_w;
 ($exitcode, $signal) = System({
    cmd             => ['fake_cmd_name', 'arg1', 'arg2'],
    real_cmd        => 'real_cmd',
    fh              => [$pipe_r, \*STDOUT, \*STDERR],
    timeout         => 5,
    cb              => sub {
        my ($pid) = @_;
        close $pipe_r;
        print {$pipe_w} "cmd stdin\n";
        close $pipe_w;
    },
 });

 use POWER::Utils qw( weak_cb );
 $cb = weak_cb($self, 'method', @params);

 use POWER::Utils qw( quote_list unquote_list unquote_hash );
 $s = quote_list( @values );
 $values_ref = unquote_list( $s );

 $s = quote_list( %hash );
 $hash_ref = unquote_hash( $s );


=head1 DESCRIPTION

Bundle of general-purpose subroutines.


=head1 EXPORTS

=over

=item :DEFAULT

clock_monotonic()
run_callback()
RandCh()
Cat()
Chomp()
Echo()
Eval()
Enc()
UnEnc()
EncUri()
UnEncUri()
System()


=back

=head1 INTERFACE

=over

=item B<clock_monotonic>()

Return: clock_gettime(CLOCK_MONOTONIC) - see L<clock_gettime(2)>.


=item B<run_callback>( CB, METHOD, ARGS )

Run callback in Perl6 style (see http://dev.perl.org/perl6/rfc/321.html).

 CB	        REQUIRED. code ref OR object OR class name
 METHOD     REQUIRED. method name for CB
 ARGS	    OPTIONAL. list with params for CB
 NOTE: METHOD required only if CB is object or class name.

Return: nothing.


=item B<weak_cb>( OBJ, METHOD [, ARGS] )

Prepare safe callback to OBJ which doesn't keep non-weaken reference to OBJ.

Return CODEREF which will call OBJ->METHOD(ARGS, @_) if OBJ is still exists.


=item B<RandCh>( LENGTH [, CHARS] )

Return: string filled by LENGTH random characters.
The random characters are taken from ("A".."Z","a".."z",0..9,"_")
or from CHARS list if specified.


=item B<Chomp>( LIST )

Return: LIST after chomp(LIST) and removing "\r" at end of lines.


=item B<Cat>( FILENAME )

Slurp FILENAME.

NOTE: See L<Perl6::Slurp> for more powerful (but slower) file slurping.

Return:
in SCALAR context contents of FILENAME in single scalar;
in LIST context all lines of FILENAME as list.


=item B<Echo>( MODE_FILENAME, LIST )

Output LIST into MODE_FILENAME.

MODE_FILENAME is used as param for open() as is and it should result in opening
file for writing. Examples of MODE_FILENAME:

 '> /tmp/file'
 '>> /tmp/file'
 '| someprog'

If LIST contain "\n" then output LIST as is. Else append "\n" to end of every
element of LIST.

Return: nothing.


=item B<Eval>( STRING [, NAME] )

Use do() instead of eval() to evaluate perl code in STRING to allow using
source filters in that code. If NAME given it will be shown in case of
errors while executing STRING.


=item B<Enc>( STRING )

DEPRECATED. Will be removed in next versions.
Use POWER::CGI::encode_html() instead.

Return: STRING after replacing unsafe HTML characters by HTML-entities.


=item B<UnEnc>( STRING )

DEPRECATED. Will be removed in next versions.
Use POWER::CGI::decode_html() instead.

Return: STRING after decoding all HTML entities.


=item B<EncUri>( STRING [, REGEXP] )

DEPRECATED. Will be removed in next versions.
Use POWER::CGI::encode_uri() instead.

Return: STRING after escaping unsafe URI characters.

REGEXP can be used to define unsafe characters in form: qr/[ABC\d\s]/ .
By default unsafe characters are defined as qr/[^\w *\@.-]/ .

=over

=item RATIONALE

If you wonder why not use standard URI::Escape::uri_escape() instead
of EncUri(), then look at this table which show how different browsers,
EncUri() and uri_escape() will encode unsafe symbols (0x20-0x7E) in url:

        Opera8  Lynx    IE6     EncUri  uri_escape
 [ ]    +       +       +       +       %20
 [!]    %21     %21     %21     %21     !
 [']    %27     %27     %27     %27     '
 [(]    %28     %28     %28     %28     (
 [)]    %29     %29     %29     %29     )
 [*]    %2A     *       *       *       *
 [-]    -       -       -       -       -
 [.]    .       .       .       .       .
 [@]    %40     @       @       @       %40
 [~]    %7E     %7E     %7E     %7E     ~
 all other symbols in [0x20-0x7E] encoded by %XX

and here is benchmark:

 perl -MBenchmark=cmpthese -MPOWER::Utils -MURI::Escape -e '
    $str = join q{}, map {chr} 0x20 .. 0x7F;
    cmpthese 100000, {
        EncUri      => q{ $a=EncUri($str)     },
        uri_escape  => q{ $b=uri_escape($str) },
    };
    '
               Rate uri_escape     EncUri
 uri_escape 14045/s         --       -17%
 EncUri     16835/s        20%         --

=back


=item B<UnEncUri>( STRING )

DEPRECATED. Will be removed in next versions.
Use POWER::CGI::decode_uri() instead.

Return: STRING after unescaping it as URI.


=item B<quote_list>( LIST )

quote_list() takes a LIST of strings, and returns a single string with the
value of each element of LIST separated from the next by a single space.
When forming the string, the text of any element that contains white space
or single quotes is first quoted by surrounding it by single quotes
('...') within which each existing single quote is doubled (''), following
the conventions of OS Inferno's sh(1).

Return: STRING containing LIST elements quoted.


=item B<unquote_list>( STRING )

unquote_list() takes a STRING, quoted according to the conventions of
quote_list(), and splits it into separate strings. It splits the string at
each maximal sequence of unquoted white space (blank, newline or tab),
stripping single quotes except where paired, to form the corresponding
list of strings, which it returns.

Return: ARRAYREF to list with elements quoted in STRING or UNDEF on error.


=item B<unquote_hash>( STRING )

Return: HASHREF to hash formed from list returned by unquote_list() or
UNDEF on error.


=item B<System>( \%opt )

Emulate system() with additional feature: flexible filehandle configuration
and ability to interact with running command using pipes.

%opt can contain these keys:

 {cmd}              - ARRAYREF  REQUIRED
        Command to execute.
        This array will be used as "LIST" param for exec().
        See `perldoc -f exec` for details.
 {real_cmd}         - SCALAR
        Real command name (if {cmd}[0] contain fake command name
        then you should put real command name into {real_cmd}).
        It will be used as "PROGRAM" param for exec().
        See `perldoc -f exec` for details.
 {fh}               - ARRAYREF
        Filehandles layout for child. If unset, then no special
        operations will be done, and child will have usual layout
        (STDIN, STDOUT, STDERR same as in parent process, plus
        any additional filehandles without close-on-exec flag).
        If set, then child will have file descriptor 0 open to
        filehandle in {fh}[0], file descriptor 1 open to {fh}[1]
        and so on for each element in {fh} array.
 {timeout}          - SCALAR
        Timeout (in seconds). If set, then alarm(timeout) will be
        executed before exec(). If exec()'ed command will not
        modify SIGALRM handler, will not set own alarm(), and
        will not inherit parent's 'IGNORE' handler for SIGALRM
        (see {inherit_signals} below), then it will receive
        SIGALRM timely and will be killed (if it will not exit()
        normally before this, of course).
 {inherit_signals}  - BOOL
        By default, command will be executed with clean signal
        state - no signals masked and all signals set to default
        handlers. But if {inherit_signals} will be true, then
        command will inherit masked and ignored signals from
        parent.
 {foreground}       - BOOL
        By default, only executed command will receive SIGINT
        and SIGQUIT if user press Ctrl-C or Ctrl-\ (parent
        will have these signal ignored until command exit).
        If {foreground} will be true, then parent handlers
        for SIGINT and SIGQUIT will not be set to 'IGNORE'
        while executing command and so parent will be able
        to receive these signals.
 {cb}               - SCALAR
 {method}           - SCALAR
        User callback. It will be executed after fork() with
        only param - child PID - and can interact with executed
        command using pipes.
        If {cb} die() then SIGKILL will be sent to command,
        parent process state will be restored and exception
        will be propagated from System().
        See L<POWER::Utils/run_callback> for details about
        {cb} and {method} format. Default {method} is 'SYSTEM'.
        WARNING! Callback will be executed in slightly modified
        environment than code which called System():
            SIGPIPE set to 'IGNORE'
            SIGCHLD blocked
            SIGINT, SIGQUIT set to 'IGNORE' (unless {foreground}
                is true)
        You can modify SIGPIPE, SIGINT and SIGQUIT handlers in
        callback, but their state will be restored after return
        from callback. Unblocking SIGCHLD in callback forbidden.

Return: command exit status $? splitted into 3-element list:
($exitcode, $signal, $core).

=over

=item PIPE EXAMPLE (a-la IPC::Open3)

 use POWER::Utils::IO;
 ### PREPARE FILEHANDLES
 my (%cmd_fh, %r_fh, %w_fh);
 pipe $r_fh{$_}, $cmd_fh{$_} or die "pipe: $!" for qw(out err);
 pipe $cmd_fh{$_}, $w_fh{$_} or die "pipe: $!" for qw(in);
 Nonblock($_) for values %r_fh, values %w_fh;
 ### INITIALIZE BUFFERS
 my (%buf);
 $buf{$_} = q{} for keys %r_fh;
 $buf{in} = 'some data for cmd STDIN';
 ### RUN COMMAND
 my ($exitcode, $signal) = System({
     cmd     => [ 'command','arg1','arg2' ],
     fh      => [ @cmd_fh{'in','out','err'} ],
     cb      => sub {
         ### CLOSE CHILD'S FILEHANDLES
         close($_) for values %cmd_fh;
         ### I/O MULTIPLEXING
         while (keys(%r_fh) + keys(%w_fh)) {
             my ($r, $w) = Select(\%r_fh, \%w_fh);
             for (@$r) {
                 my $is_eof = Sysread($r_fh{$_}, $buf{$_});
                 delete $r_fh{$_} if $is_eof;
             }
             for (@$w) {
                 my $done = eval {Syswrite($w_fh{$_}, $buf{$_})};
                 die "[$_]: $@"   if $@ && $@ !~ /Broken pipe/;
                 delete $w_fh{$_} if $@ || $done;
             }
         }
     },
 });
 ### PROCESS RESULT
 warn "Command doesn't read all data from STDIN?!"
    if length $buf{in};
 print "Command STDOUT:\n" . $buf{out};
 warn  "Command STDERR:\n" . $buf{err};
 if ($signal) {
    warn  "Command was killed by signal $signal\n";
 } else {
    print "Command exit code is $exitcode\n";
 }


=item RATIONALE

There already exists a lot of ways to execute command: system(), open(),
`` (backticks), IPC::Open3, IPC::Run, IPC::Run3. In short,
system() is good but doesn't allow interaction with running command,
and all others doesn't handle signals correctly. Below is gore details,
if interested.

While it's possible with system() to run command with custom filehandles
instead of inheriting only STDIN/STDOUT/STDERR of current process, it has
some limitations:

 - You can't use safe LIST form of system() and should execute
   your command using shell to have filehandle redirections
   like '2>&1' working.
 - You can't interact with running command using pipes.
 - If you need to give your non-STDIN/OUT/ERR filehandle or if
   you need to give more than 3 filehandles to command you should
   use fcntl() to modify close-on-exec flag and/or reopen your
   STDIN/OUT/ERR to needed filehandles.
 - User can't set timeout for command. There no way to setup
   alarm() for command. User can setup alarm() for his main
   process, but:
   1) this isn't acceptable in module because user can already
      setup some alarm() before calling our module
   2) this alarm() will interrupt system(), but not interrupt
      running command (and we can't kill it because we don't
      know it's pid)

open() limitations:

 - open(), unlike system(), doesn't block SIGCHLD. This may be
   fine in user code, but in module this result in two problems
   if user has $SIG{CHLD} handler installed:
   1) pid of process open()'ed in module will be delivered into
      user's SIGCHLD handler
   2) $? status will not be available in module
 - open(), unlike system(), doesn't block SIGINT & SIGQUIT in
   main process so if you use open() instead of system() just
   to interact with executed command using pipe like:
        open(my $fh, 'some_foreground_command |')
   then both this command and your main process will receive
   these signals if user press Ctrl-C or Ctrl-\.
 - If command will exit while user print() into pipe SIGPIPE
   will kill main process if user don't block it.
 - All system() limitations also apply to open(), with only
   correction: you can interact with running command using
   pipe, but only single pipe - if you need more pipes then
   you should use IPC::Open3, IPC::Run or IPC::Run3.

`command` (backticks) limitations:

 - No safe LIST form.
 - You can't interact with running command using pipes.
 - If you need to give your non-STDIN/OUT/ERR filehandle or if
   you need to give more than 3 filehandles to command you should
   use fcntl() to modify close-on-exec flag and/or reopen your
   STDIN/OUT/ERR to needed filehandles.
 - All open() limitations apply.

IPC::Open3 limitations:

 - No signal handling at all, so all open() limitations about
   SIGCHLD, SIGINT, SIGQUIT and SIGPIPE apply.
 - No timeout.
 - Unable to use more than 3 filehandles.

IPC::Run limitations:

 - No signal handling at all, so all open() limitations about
   SIGCHLD, SIGINT, SIGQUIT and SIGPIPE apply.

IPC::Run3 limitations:

 - No signal handling at all, so all open() limitations about
   SIGCHLD, SIGINT, SIGQUIT and SIGPIPE apply.
 - You can't interact with running command using pipes.


=back

=back


=head1 DIAGNOSTICS

=head2 EXCEPTIONS

=over

=item C<< clock_gettime: $! >>

clock_monotonic() got error in low-level L<clock_gettime(3)>.

=item C<< usage: run_callback( CB [, METHOD [, @ARGS]] ) >>

run_callback() was executed without params.

=item C<< run_callback: wrong CB ... >>

First param of run_callback() isn't one of:

 CODE ref
 OBJECT
 CLASS name

=item C<< usage: RandCh( LENGTH [, CHARS] ) >>

RandCh() was executed without params.

=item C<< RandCh: LENGTH must be positive >>

First RandCh() param must be a positive number.

=item C<< usage: Cat( FILENAME ) >>

Cat() must be executed with single param.

=item C<< open: $! >>

This exception can happens in Cat() , Echo() and Eval() .
In case of Cat() and Echo() it mean you give them wrong FILENAME param.
In case of Eval() ... maybe you try to evaluate something strange?

=item C<< usage: Echo( MODE_FILENAME, LIST ) >>

Echo() require at least two params.

=item C<< Echo: Bad file descriptor >>

Echo() unable to write into MODE_FILENAME.

=item C<< usage: Chomp( LIST ) >>

Chomp() require at least one param.

=item C<< usage: Eval( SCALAR [, EVALNAME] ) >>

Eval() require one or two params.

=item C<< usage: Enc( SCALAR ) >>
=item C<< Enc: $_[0] undefined >>

Enc() require one param and it should be defined.

=item C<< usage: UnEnc( SCALAR ) >>
=item C<< UnEnc: $_[0] undefined >>

UnEnc() require one param and it should be defined.

=item C<< usage: EncUri( SCALAR [, REGEXP] ) >>

EncUri() require one or two params.

=item C<< EncUri: $_[0] undefined >>

Guess? Yeah, first EncUri() param must be defined.

=item C<< usage: UnEncUri( SCALAR ) >>
=item C<< UnEncUri: $_[0] undefined >>

UnEncUri() require one param and it should be defined.


=back

=head2 WARNINGS

None.

=head1 CONFIGURATION AND ENVIRONMENT

POWER::Utils requires no configuration files or environment variables.


=head1 DEPENDENCIES

 version
 Perl6::Export::Attrs
 Inline


=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-utils@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005, Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
